@@ -1,16 +1,16 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
+import contextlib
 from pathlib import Path
 import warnings
-import country_converter as coco
 import geopandas as gpd
 from matplotlib import pyplot as plt
 import pandas as pd
 from covsirphy.util.argument import find_args
 from covsirphy.util.error import deprecate, SubsetNotFoundError, UnExpectedValueError
 from covsirphy.util.term import Term
-from covsirphy.visualization.colored_map import ColoredMap
+from covsirphy._deprecated.colored_map import ColoredMap
 
 
 class CleaningBase(Term):
@@ -24,8 +24,8 @@ class CleaningBase(Term):
                 reset index
             Columns
                 - Date: Observation date
-                - ISO3: ISO3 code (optional)
-                - Country: country/region name
+                - ISO3: ISO3 code
+                - Country: country/region name (optional)
                 - Province: province/prefecture/state name
                 - Confirmed: the number of confirmed cases
                 - Fatal: the number of fatal cases
@@ -38,9 +38,9 @@ class CleaningBase(Term):
         Either @filename (high priority) or @data must be specified.
 
     Note:
-        - If @filename is None, geometry information will be saved in "input" directory.
-        - If @filename is not None, geometry information will be saved in the directory which has the file.
-        - The directory of geometry information could be changed with .directory property.
+        - If @filename is None, geography information will be saved in "input" directory.
+        - If @filename is not None, geography information will be saved in the directory which has the file.
+        - The directory of geography information could be changed with .directory property.
     """
 
     def __init__(self, filename=None, data=None, citation=None, variables=None):
@@ -96,6 +96,114 @@ class CleaningBase(Term):
         }
         return pd.read_csv(filename, dtype=dtype_dict).reindex(columns=columns)
 
+    @staticmethod
+    def _ensure_instance(target, class_obj, name="target"):
+        """
+        Ensure the target is an instance of the class object.
+
+        Args:
+            target (instance): target to ensure
+            parent (class): class object
+            name (str): argument name of the target
+
+        Returns:
+            object: as-is target
+        """
+        s = f"@{name} must be an instance of {class_obj}, but {type(target)} was applied."
+        if not isinstance(target, class_obj):
+            raise TypeError(s)
+        return target
+
+    @staticmethod
+    def _ensure_dataframe(target, name="df", time_index=False, columns=None, empty_ok=True):
+        """
+        Ensure the dataframe has the columns.
+
+        Args:
+            target (pandas.DataFrame): the dataframe to ensure
+            name (str): argument name of the dataframe
+            time_index (bool): if True, the dataframe must has DatetimeIndex
+            columns (list[str] or None): the columns the dataframe must have
+            empty_ok (bool): whether give permission to empty dataframe or not
+
+        Returns:
+            pandas.DataFrame:
+                Index
+                    as-is
+                Columns:
+                    columns specified with @columns or all columns of @target (when @columns is None)
+        """
+        if not isinstance(target, pd.DataFrame):
+            raise TypeError(f"@{name} must be a instance of (pandas.DataFrame).")
+        df = target.copy()
+        if time_index and (not isinstance(df.index, pd.DatetimeIndex)):
+            raise TypeError(f"Index of @{name} must be <pd.DatetimeIndex>.")
+        if not empty_ok and target.empty:
+            raise ValueError(f"@{name} must not be a empty dataframe.")
+        if columns is None:
+            return df
+        if not set(columns).issubset(df.columns):
+            cols_str = ", ".join(col for col in columns if col not in df.columns)
+            included = ", ".join(df.columns.tolist())
+            s1 = f"Expected columns were not included in {name} with {included}."
+            raise KeyError(f"{s1} {cols_str} must be included.")
+        return df.loc[:, columns]
+
+    @staticmethod
+    def _ensure_natural_int(target, name="number", include_zero=False, none_ok=False):
+        """
+        Ensure a natural (non-negative) number.
+
+        Args:
+            target (int or float or str or None): value to ensure
+            name (str): argument name of the value
+            include_zero (bool): include 0 or not
+            none_ok (bool): None value can be applied or not.
+
+        Returns:
+            int: as-is the target
+
+        Note:
+            When @target is None and @none_ok is True, None will be returned.
+            If the value is a natural number and the type was float or string,
+            it will be converted to an integer.
+        """
+        if target is None and none_ok:
+            return None
+        s = f"@{name} must be a natural number, but {target} was applied"
+        try:
+            number = int(target)
+        except TypeError as e:
+            raise TypeError(f"{s} and not converted to integer.") from e
+        if number != target:
+            raise ValueError(f"{s}. |{target} - {number}| > 0")
+        min_value = 0 if include_zero else 1
+        if number < min_value:
+            raise ValueError(f"{s}. This value is under {min_value}")
+        return number
+
+    @classmethod
+    def _ensure_date(cls, target, name="date", default=None):
+        """
+        Ensure the format of the string.
+
+        Args:
+            target (str or pandas.Timestamp): string to ensure
+            name (str): argument name of the string
+            default (pandas.Timestamp or None): default value to return
+
+        Returns:
+            pandas.Timestamp or None: as-is the target or default value
+        """
+        if target is None:
+            return default
+        if isinstance(target, pd.Timestamp):
+            return target.replace(hour=0, minute=0, second=0, microsecond=0)
+        try:
+            return pd.to_datetime(target).replace(hour=0, minute=0, second=0, microsecond=0)
+        except ValueError as e:
+            raise ValueError(f"{name} was not recognized as a date, {target} was applied.") from e
+
     @property
     def raw(self):
         """
@@ -113,7 +221,7 @@ class CleaningBase(Term):
     @property
     def directory(self):
         """
-        str: directory name to save geometry information
+        str: directory name to save geography information
         """
         return str(self._dirpath)
 
@@ -183,35 +291,24 @@ class CleaningBase(Term):
             errors (str): 'raise' or 'coerce'
 
         Returns:
-            str: country name
+            str: ISO3 code
 
         Raises:
             SubsetNotFoundError: no records were found for the country and @errors is 'raise'
         """
+        iso3 = self._to_iso3(country)[0]
         df = self._cleaned_df.copy()
+        if self.ISO3 in df and iso3 in df[self.ISO3].unique():
+            return iso3
         self._ensure_dataframe(df, name="the cleaned dataset", columns=[self.COUNTRY])
         selectable_set = set(df[self.COUNTRY].unique())
         # return country name as-is if selectable
         if country in selectable_set:
-            return country
-        # Convert country name
-        warnings.simplefilter("ignore", FutureWarning)
-        converted = coco.convert(country, to="name_short", not_found=None)
-        # Additional abbr
-        abbr_dict = {
-            "Congo Republic": "Republic of the Congo",
-            "DR Congo": "Democratic Republic of the Congo",
-            "UK": "United Kingdom",
-            "Vatican": "Holy See",
-        }
-        name = abbr_dict.get(converted, converted)
-        # Return the name if registered in the dataset
-        if name in selectable_set:
-            return name
+            return iso3
         if errors == "raise":
-            raise SubsetNotFoundError(country=country, country_alias=name)
+            raise SubsetNotFoundError(country=country, country_alias=iso3)
 
-    @deprecate("CleaningBase.iso3_to_country()", new="CleaningBase.ensure_country_name()")
+    @deprecate("CleaningBase.iso3_to_country()")
     def iso3_to_country(self, iso3_code):
         """
         Convert ISO3 code to country name if records are available.
@@ -227,6 +324,7 @@ class CleaningBase(Term):
         """
         return self.ensure_country_name(iso3_code)
 
+    @deprecate("CleaningBase.country_to_iso3()", new="CleaningBase.ensure_country_name()", version="2.24.0-gamma")
     def country_to_iso3(self, country, check_data=True):
         """
         Convert country name to ISO3 code if records are available.
@@ -242,12 +340,13 @@ class CleaningBase(Term):
             str: ISO3 code or "---" (when unknown)
         """
         name = self.ensure_country_name(country) if check_data else country
-        return coco.convert(name, to="ISO3", not_found="---")
+        iso3 = self._to_iso3(name)[0]
+        return self.NA * 3 if iso3 is None or iso3 == country else iso3
 
     @classmethod
     def area_name(cls, country, province=None):
         """
-        Return area name of the country/province.
+        Return area name of the province/country.
 
         Args:
             country (str): country name or ISO3 code
@@ -258,11 +357,11 @@ class CleaningBase(Term):
 
         Note:
             If province is None or '-', return country name.
-            If not, return the area name, like 'Japan/Tokyo'
+            If not, return the area name, like 'Tokyo/Japan'
         """
-        if province in [None, cls.UNKNOWN]:
+        if province in [None, cls.NA]:
             return country
-        return f"{country}{cls.SEP}{province}"
+        return f"{province}{cls.SEP}{country}"
 
     def layer(self, country=None):
         """
@@ -289,20 +388,21 @@ class CleaningBase(Term):
             When @country is a country name, province level data in the selected country will be returned.
         """
         df = self._cleaned_df.copy()
-        self._ensure_dataframe(df, name="the cleaned dataset", columns=[self.COUNTRY])
-        if self.PROVINCE not in df:
-            df[self.PROVINCE] = self.UNKNOWN
-        df[self.AREA_COLUMNS] = df[self.AREA_COLUMNS].astype(str)
+        for col in [self.ISO3, self.COUNTRY, self.PROVINCE]:
+            df[col] = df[col].astype(str, errors="ignore")
         # Country level data
         if country is None:
-            df = df.loc[df[self.PROVINCE] == self.UNKNOWN]
-            return df.drop(self.PROVINCE, axis=1).reset_index(drop=True)
+            if self.PROVINCE in df:
+                df = df.loc[df[self.PROVINCE] == self.NA].drop(self.PROVINCE, axis=1)
+            return df.reset_index(drop=True)
         # Province level data at the selected country
-        country_alias = self.ensure_country_name(country, errors="coerce")
-        df = df.loc[df[self.COUNTRY] == country_alias]
+        self._ensure_dataframe(df, name="the cleaned dataset", columns=[self.PROVINCE])
+        iso3 = self.ensure_country_name(country)
+        df = df.loc[df[self.ISO3] == iso3]
         if df.empty:
-            raise SubsetNotFoundError(country=country, country_alias=country_alias) from None
-        df = df.loc[df[self.PROVINCE] != self.UNKNOWN]
+            raise SubsetNotFoundError(country=country, country_alias=iso3) from None
+        if self.PROVINCE in df:
+            df = df.loc[df[self.PROVINCE] != self.NA]
         return df.reset_index(drop=True)
 
     def _subset_by_area(self, country, province=None):
@@ -320,10 +420,10 @@ class CleaningBase(Term):
             SubsetNotFoundError: no records were found for the condition
         """
         # Country level
-        if province is None or province == self.UNKNOWN:
+        if province is None or province == self.NA:
             df = self.layer(country=None)
-            country_alias = self.ensure_country_name(country)
-            df = df.loc[df[self.COUNTRY] == country_alias]
+            iso3 = self.ensure_country_name(country)
+            df = df.loc[df[self.ISO3] == iso3]
             return df.reset_index(drop=True)
         # Province level
         df = self.layer(country=country)
@@ -352,12 +452,12 @@ class CleaningBase(Term):
         Raises:
             SubsetNotFoundError: no records were found for the condition
         """
-        country_alias = self.ensure_country_name(country, errors="coerce")
+        iso3 = self.ensure_country_name(country, errors="coerce")
         try:
             df = self._subset_by_area(country=country, province=province)
         except SubsetNotFoundError:
             raise SubsetNotFoundError(
-                country=country, country_alias=country_alias, province=province) from None
+                country=country, country_alias=iso3, province=province) from None
         df = df.drop([self.COUNTRY, self.ISO3, self.PROVINCE], axis=1, errors="ignore")
         # Subset with Start/end date
         if start_date is None and end_date is None:
@@ -369,13 +469,13 @@ class CleaningBase(Term):
         df = df.loc[(start_obj <= series) & (series <= end_obj), :]
         if df.empty:
             raise SubsetNotFoundError(
-                country=country, country_alias=country_alias, province=province,
+                country=country, country_alias=iso3, province=province,
                 start_date=start_date, end_date=end_date) from None
         return df.reset_index(drop=True)
 
     def subset_complement(self, country, **kwargs):
         """
-        Return the subset. If necessary, complemention will be performed.
+        Return the subset. If necessary, complementing will be performed.
 
         Raises:
             NotImplementedError
@@ -385,7 +485,7 @@ class CleaningBase(Term):
     def records(self, country, province=None, start_date=None, end_date=None,
                 auto_complement=True, **kwargs):
         """
-        Return the subset. If necessary, complemention will be performed.
+        Return the subset. If necessary, complementing will be performed.
 
         Args:
             country (str): country name or ISO3 code
@@ -402,22 +502,20 @@ class CleaningBase(Term):
                 Columns
                     without ISO3, Country, Province column
         """
-        country_alias = self.ensure_country_name(country)
+        iso3 = self.ensure_country_name(country)
         subset_arg_dict = {
             "country": country, "province": province, "start_date": start_date, "end_date": end_date}
         if auto_complement:
-            try:
+            with contextlib.suppress(NotImplementedError):
                 df, is_complemented = self.subset_complement(
                     **subset_arg_dict, **kwargs)
                 if not df.empty:
                     return (df, is_complemented)
-            except NotImplementedError:
-                pass
         try:
             return (self.subset(**subset_arg_dict), False)
         except SubsetNotFoundError:
             raise SubsetNotFoundError(
-                country=country, country_alias=country_alias, province=province,
+                country=country, country_alias=iso3, province=province,
                 start_date=start_date, end_date=end_date) from None
 
     def countries(self):
@@ -472,15 +570,13 @@ class CleaningBase(Term):
         df = df.loc[df[self.COUNTRY] != self.OTHERS]
         # Recognize province as a region/country
         if self.PROVINCE in df:
-            try:
+            with contextlib.suppress(ValueError):
                 df[self.ISO3] = df[self.ISO3].cat.add_categories(["GRL"])
                 df[self.COUNTRY] = df[self.COUNTRY].cat.add_categories(["Greenland"])
-                df.loc[df[self.PROVINCE] == "Greenland", self.AREA_ABBR_COLS] = ["GRL", "Greenland", self.UNKNOWN]
-            except ValueError:
-                pass
+                df.loc[df[self.PROVINCE] == "Greenland", self.AREA_ABBR_COLS] = ["GRL", "Greenland", self.NA]
         # Select country level data
         if self.PROVINCE in df.columns:
-            df = df.loc[df[self.PROVINCE] == self.UNKNOWN]
+            df = df.loc[df[self.PROVINCE] == self.NA]
         # Select date
         if date is not None:
             self._ensure_dataframe(df, name="cleaned dataset", columns=[self.DATE])
@@ -503,24 +599,24 @@ class CleaningBase(Term):
             kwargs: arguments of covsirphy.ColoredMap() and covsirphy.ColoredMap.plot()
         """
         df = self._cleaned_df.copy()
-        country_alias = self.ensure_country_name(country)
+        iso3 = self.ensure_country_name(country)
         # Check variable name
         if variable not in df.columns:
             candidates = [col for col in df.columns if col not in self.AREA_ABBR_COLS]
             raise UnExpectedValueError(name="variable", value=variable, candidates=candidates)
         # Select country-specific data
-        self._ensure_dataframe(df, name="cleaned dataset", columns=[self.COUNTRY, self.PROVINCE])
-        df = df.loc[df[self.COUNTRY] == country_alias]
-        df = df.loc[df[self.PROVINCE] != self.UNKNOWN]
+        self._ensure_dataframe(df, name="cleaned dataset", columns=[self.ISO3, self.PROVINCE])
+        df = df.loc[df[self.ISO3] == iso3]
+        df = df.loc[df[self.PROVINCE] != self.NA]
         if df.empty:
             raise SubsetNotFoundError(
-                country=country, country_alias=country_alias, message="at province level")
+                country=country, country_alias=iso3, details="at province level")
         # Select date
         if date is not None:
             self._ensure_dataframe(df, name="cleaned dataset", columns=[self.DATE])
             df = df.loc[df[self.DATE] == pd.to_datetime(date)]
         df = df.groupby(self.PROVINCE).last().reset_index()
         # Plotting
-        df[self.COUNTRY] = country_alias
+        df[self.COUNTRY] = country
         df.rename(columns={variable: "Value"}, inplace=True)
         self._colored_map(title=title, data=df, level=self.PROVINCE, **kwargs)
